function besa2brainstorm(besa_channels, bstVar, bWithChannels)
% besa2brainstorm   Import the besa_channels structure to Brainstorm.
%
% Parameters:
%	[besa_channels]
%       BESA besa_channels data structure
%
%   [bstVar]
%       A structure for the Brainstorm database
%           bstVar.ProtocolName : protocol name
%           bstVar.SubjectName  : subject name
%           bstVar.StudyName    : study name
%
%   [bWithChannels]
%       true : with channel information (default)
%       false: withouth channel information
%
% Example:
%   bstVar.ProtocolName = 'Protocol01';
%   bstVar.SubjectName  = 'Subject01';
%   bstVar.StudyName    = 'Data01';
%   bWithChannels       = true;
%   besa2brainstorm(besa_channels_2events, bstVar, bWithChannels);
%
% See also:
%   https://neuroimage.usc.edu/brainstorm/Tutorials/Scripting#Example:_Creating_a_new_file
%   https://neuroimage.usc.edu/brainstorm/Tutorials/Epoching#Import_in_database
%   https://neuroimage.usc.edu/brainstorm/Tutorials/Epoching#On_the_hard_drive
%   https://neuroimage.usc.edu/brainstorm/Tutorials/Averaging#On_the_hard_drive

% Copyright (C) 2021, BESA GmbH
%
% This file is part of BESA2MATLAB.
%
%    BESA2MATLAB is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    BESA2MATLAB is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with BESA2MATLAB. If not, see <http://www.gnu.org/licenses/>.
%
% Author: Jae-Hyun Cho
% Created: 2021-01-25

%{
%% Example

% Add paths: BESA2MATLAB; Brainstorm
%addpath('xxx');
%addpath('xxx');

% Epoched data
bstVar.ProtocolName = 'Protocol01';
bstVar.SubjectName  = 'Subject01';
bstVar.StudyName    = 'test_S1_EpochedData';
besa2brainstorm(besa_channels_events, bstVar, true);

% Raw data
bstVar.ProtocolName = 'Protocol01';
bstVar.SubjectName  = 'Subject01';
bstVar.StudyName    = 'test_S1_RawData';
besa2brainstorm(besa_channels_raw, bstVar, true);

brainstorm
%}

% Set default values
if nargin < 2
    error('Wrong input arguments');
elseif nargin < 3
    bWithChannels = true;
end

% Check channel type
if isfield(besa_channels, 'channeltypes')    
    chTypeAll = [besa_channels.channeltypes];
    chType = unique(chTypeAll);
    if sum(strcmp(chType,'EEG')) == 0
        % TODO: MEG
        error('Support only EEG channel type');
    end
else
    error('The besa_channels.channeltypes is missing');
end

% Check brainstorm
if exist('brainstorm', 'file')
    % Start brainstorm
    if ~brainstorm('status')
        brainstorm nogui
    end
else
    error('Add brainstorm to the search path');
end

% Get the protocol index
ProtocolName = bstVar.ProtocolName;
iProtocol = bst_get('Protocol', ProtocolName);
if isempty(iProtocol)
    error(['Unknown protocol: ' ProtocolName]);
end

% Select the current procotol
gui_brainstorm('SetCurrentProtocol', iProtocol);

% Exported data from BESA (besa_channels) -> Brainstorm
if strcmp(besa_channels.datatype, 'Raw_Data')      
    % Raw data -> Brainstorm
    add_BESA_RawData(besa_channels, bstVar, bWithChannels);
elseif (strcmp(besa_channels.datatype, 'Epoched_Data') || ...
        strcmp(besa_channels.datatype, 'Segment'))
    % Epoched or Segment data -> Brainstorm
    add_BESA_EpochedData(besa_channels, bstVar, bWithChannels);
else
    error('The besa_channels.datatype is wrong');

end
    
end

% =========================================================================

function scale = scaleUnit(channelUnit)
    % TODO: MEG (fT)
    
    if strcmp(channelUnit, 'V')
        scale = 1e-6;
    elseif strcmp(channelUnit, 'V')
        scale = 1;
    else
        scale = 1e-6;
    end
end

% -------------------------------------------------------------------------

function scale = scaleTime()
    scale = 1e-3;
end

% -------------------------------------------------------------------------

function channels = convertChannel(besa_channels)

    nCh = length(besa_channels.channeltypes);
    
    channels.Comment    = 'BESA_MATLAB channels';
    channels.HeadPoints = [];
    channels.Channel    = [];
    
    for i = 1:nCh
        
        % EOG and ECG
        channelType = besa_channels.channeltypes{i};
        if strncmp(channelType, 'POLYGR', 6)
            if strncmp(besa_channels.channellabels{i}, 'EOG', 3)
                channelType = 'EOG';
            elseif strncmp(besa_channels.channellabels{i}, 'ECG', 3)
                channelType = 'ECG';
            end
        end
        
        channels.Channel(i).Name     = besa_channels.channellabels{i};
        channels.Channel(i).Comment  = [];
        channels.Channel(i).Type     = channelType;
        channels.Channel(i).Group    = [];
        channels.Channel(i).Loc(1,1) = transpose(besa_channels.channelcoordinates(i,2)); % X
        channels.Channel(i).Loc(2,1) = transpose(besa_channels.channelcoordinates(i,1)); % Y
        channels.Channel(i).Loc(3,1) = transpose(besa_channels.channelcoordinates(i,3)); % Z
        channels.Channel(i).Orient   = [];
        channels.Channel(i).Weight   = 1;
    end
    
end

% -------------------------------------------------------------------------

function add_BESA_RawData(besa_channels, bstVar, bWithChannels)

    % Create a new folder "bstVar.StudyName" in subject "bstVar.SubjectName"
    iStudy = db_add_condition(bstVar.SubjectName, bstVar.StudyName);        
    
    % Get the corresponding study structure
    sStudy = bst_get('Study', iStudy);
    
    % Get the full path to the new folder
    % (same folder as the brainstormstudy.mat file for this study)
    OutputFolder = bst_fileparts(file_fullpath(sStudy.FileName));
    
    % ===== Channels =====
    if bWithChannels
        if (isfield(besa_channels, 'channellabels') && ...
                isfield(besa_channels, 'channeltypes') && ...
                isfield(besa_channels, 'channelcoordinates'))

            channels = convertChannel(besa_channels);

            % Generate a unique filename (with a timestamp)
            MatrixFile = bst_process('GetNewFilename', OutputFolder, 'channel');
            
            % Save file
            bst_save(MatrixFile, channels, 'v6');
            
            % Reference saved file in the database
            db_add_data(iStudy, MatrixFile, channels);
        else
            error('Error occurred.');
        end
    end
    
    % ===== Raw data =====
    nCh = numel(besa_channels.channeltypes);
    
    data = db_template('datamat');
    %data.F           = [];
    %data.Std         = [];
    %data.Comment     = [];
    data.ChannelFlag = ones(nCh,1);
    %data.Time        = [];
    %data.DataType    = 'recordings';
    data.Device      = 'BESA_MATLAB';
    data.nAvg        = 1;
    data.Leff        = 1;
    %data.Events      = [];
    %data.History     = [];

    if strcmp(besa_channels.datatype, 'Raw_Data')
        
        nTime = size(besa_channels.amplitudes, 1);
        
        data.Comment = 'Raw_Data';
        data.F       = zeros(nCh, nTime);
        data.Time    = scaleTime() .* besa_channels.latencies;

        for iCh = 1:nCh
            data.F(iCh,:) = scaleUnit(besa_channels.channelunits{iCh}) .* ...
                transpose(besa_channels.amplitudes(:, iCh));
        end
        
        data.Events = convertEvents(data.Events, besa_channels.events);
    else
        error('The besa_channels.datatype should be Raw_Data');
    end
    
    % Generate a unique filename (with a timestamp)
    MatrixFile = bst_process('GetNewFilename', OutputFolder, 'data');
    
    % Save file
    bst_save(MatrixFile, data, 'v6');
    
    % Reference saved file in the database
    db_add_data(iStudy, MatrixFile, data);
    
    % Update the database explorer display
    %panel_protocols('UpdateNode', 'Study', iStudy);
    
    % Reload only the select data folder
    db_reload_studies(iStudy);
end

% -------------------------------------------------------------------------

function add_BESA_EpochedData(besa_channels, bstVar, bWithChannels)
    % Create a new folder "bstVar.StudyName" in subject "bstVar.SubjectName"
    iStudy = db_add_condition(bstVar.SubjectName, bstVar.StudyName);      
    
    % Get the corresponding study structure
    sStudy = bst_get('Study', iStudy);    
    
    % Get the full path to the new folder 
    % (same folder as the brainstormstudy.mat file for this study)
    OutputFolder = bst_fileparts(file_fullpath(sStudy.FileName));
    
     % ===== Channels =====
    if bWithChannels
        if (isfield(besa_channels, 'channellabels') && ...
                isfield(besa_channels, 'channeltypes') && ...
                isfield(besa_channels, 'channelcoordinates'))

            channels = convertChannel(besa_channels);

            % Generate a unique filename (without a timestamp)
            MatrixFile = bst_process('GetNewFilename', OutputFolder, 'channel');
            % Save file
            bst_save(MatrixFile, channels, 'v6');
            % Reference saved file in the database
            db_add_data(iStudy, MatrixFile, channels);
        else
            error('Error occurred.');
        end
    end
    
     % ===== Epochs or Segment =====
    if (strcmp(besa_channels.datatype, 'Epoched_Data') || ...
            strcmp(besa_channels.datatype, 'Segment'))
        
        nCh     = numel(besa_channels.channeltypes);
        nEpochs = numel(besa_channels.data);
        
        labelAll = cell(1,nEpochs);
        for iEpoch = 1:nEpochs 
            nEvent = numel(besa_channels.data(iEpoch).event);
            if nEvent == 1
                iEvent = 1;
            elseif nEvent > 1
                [~, minIdx] = min(abs([besa_channels.data(iEpoch).event.latency]));
                iEvent = minIdx;         
            else
                error('Error occurred.');
            end
            
            labelAll{iEpoch} = besa_channels.data(iEpoch).event(iEvent).label{1};
        end
        
        labels   = unique(labelAll(:));
        nLabels  = numel(labels);
        
        for iLabel = 1:nLabels            
            
            allIndex = find(strcmp(labelAll, labels{iLabel}));            
            if size(allIndex,1) > size(allIndex,2)
                allIndex = transpose(allIndex);
            end
            
            cnt = 1;
            for iIdx = allIndex
                
                nTime = size(besa_channels.data(iIdx).amplitudes, 1);
                
                data = db_template('datamat');
                data.F            = zeros(nCh, nTime);
                %data.Std         = [];
                data.Comment     = [labels{iLabel}, ' (#', num2str(cnt), ')'];
                data.ChannelFlag = ones(nCh, 1);
                data.Time        = scaleTime() .* besa_channels.data(iIdx).latencies;
                %data.DataType    = 'recordings';
                data.Device      = 'BESA_MATLAB';
                %data.nAvg        = 1;
                %data.Leff        = 1;
                %data.Events      = [];
                %data.History     = [];
                
                for iCh = 1:nCh
                    data.F(iCh,:) = scaleUnit(besa_channels.channelunits{iCh}) .* ...
                        transpose(besa_channels.data(iIdx).amplitudes(:,iCh));
                end

                data.Events = convertEvents(data.Events, besa_channels.data(iIdx).event);
               
                % Check if labels{iLabel} is valid as a filename
                fName_check = regexp(labels{iLabel}, '[/\*:?"<>|]', 'once');
                if ~isempty(fName_check)
                    fName_label = strrep(labels{iLabel}, ...
                        labels{iLabel}(fName_check), '_');
                else
                    fName_label = labels{iLabel};
                end
                                
                % Generate a unique filename
                fName = sprintf('data_%s_trial%03i.mat', fName_label, cnt);
                MatrixFile = file_unique(bst_fullfile(OutputFolder, fName));
                
                % Save file
                bst_save(MatrixFile, data, 'v6');
                
                % Reference saved file in the database
                db_add_data(iStudy, MatrixFile, data);     
                
                cnt = cnt + 1;
            end
            
        end
        
        % Update the database explorer display
        %panel_protocols('UpdateNode', 'Study', iStudy);
    else
       error('Error occurred.');
    end

    % Reload only the select data folder
    db_reload_studies(iStudy);
    
end

% -------------------------------------------------------------------------

function [brainstorm_Events] = convertEvents(brainstorm_Events, besa_events)

    % see also: brainstorm3\toolbox\io\import_events.m
    ColorTable = ...
        [0     1    0   
        .4    .4    1
         1    .6    0
         0     1    1
        .56   .01  .91
         0    .5    0
        .4     0    0
         1     0    1
        .02   .02   1
        .5    .5   .5];
    
    labelAll = [besa_events.label];
    labels   = unique(labelAll);
    nLabels  = length(labels);

    for i = 1:nLabels
        index  = find(strcmp(labelAll, labels{i}));
        nIndex = length(index);

        % If all the colors of the color table are taken, attribute colors cyclically
        iColor = mod(i-1, length(ColorTable)) + 1;

        brainstorm_Events(i).label      = labels{i};
        brainstorm_Events(i).color      = ColorTable(iColor,:);
        brainstorm_Events(i).epochs     = 1;
        brainstorm_Events(i).times      = zeros(1, nIndex);
        brainstorm_Events(i).reactTimes = [];
        brainstorm_Events(i).select     = 1;
        brainstorm_Events(i).epochs     = ones(1, nIndex);
        brainstorm_Events(i).channels   = cell(1, nIndex);
        brainstorm_Events(i).notes      = cell(1, nIndex);

        cnt = 1;
        for iEvent = index            
            brainstorm_Events(i).times(1,cnt) = scaleTime() * besa_events(iEvent).latency;
            cnt = cnt + 1;
        end
    end
    
end
